﻿using CSHTML5;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml.Controls;

namespace ThreeJS
{
    public class Viewport3D : Canvas
    {

        // Remember whether the JS library was loaded in order not to load it twice:
        static bool JSLibraryWasLoaded;

        public Viewport3D()
        {
            this.Loaded += Viewport3D_Loaded;

        }

        async void Viewport3D_Loaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // Load the JavaScript libraries if not already loaded:
            if (!JSLibraryWasLoaded)
            {
                await Interop.LoadJavaScriptFile("ms-appx:///ThreeJS/JS/three.js");
                await Interop.LoadJavaScriptFile("ms-appx:///ThreeJS/JS/Projector.js");
                await Interop.LoadJavaScriptFile("ms-appx:///ThreeJS/JS/CanvasRenderer.js");
                JSLibraryWasLoaded = true;
            }

            // Get a reference to the HTML DOM representation of the control (must be in the Visual Tree):
            object div = Interop.GetDiv(this);

            string script =
@"
        var container;

        var camera, scene, renderer;

        var cube, plane;

        var targetRotation = 0;
        var targetRotationOnMouseDown = 0;

        var mousedown = false;
        var mouseX = 0;
        var mouseXOnMouseDown = 0;
        var windowHalfX = window.innerWidth / 2;
        var windowHalfY = window.innerHeight / 2;

        init();
        
        render();

        function init() {

            container = $0;

            camera = new THREE.PerspectiveCamera(70, window.innerWidth / window.innerHeight, 1, 1000);
            camera.position.y = 150;
            camera.position.z = 500;

            scene = new THREE.Scene();
            scene.background = new THREE.Color(0xf0f0f0);

            var geometry = new THREE.BoxGeometry(200, 200, 200);

            for (var i = 0; i < geometry.faces.length; i += 2) {
                var hex = Math.random() * 0xffffff;
                geometry.faces[i].color.setHex(hex);
                geometry.faces[i + 1].color.setHex(hex);
            }

            var material = new THREE.MeshBasicMaterial({ vertexColors: THREE.FaceColors, overdraw: 0.5 });

            cube = new THREE.Mesh(geometry, material);
            cube.position.y = 150;
            scene.add(cube);

            var geometry = new THREE.PlaneBufferGeometry(200, 200);
            geometry.rotateX(- Math.PI / 2);

            var material = new THREE.MeshBasicMaterial({ color: 0xe0e0e0, overdraw: 0.5 });

            plane = new THREE.Mesh(geometry, material);
            scene.add(plane);

            renderer = new THREE.CanvasRenderer();

            renderer.setPixelRatio(window.devicePixelRatio);
            renderer.setSize(window.innerWidth, window.innerHeight);

            container.appendChild(renderer.domElement);

            document.addEventListener('mousedown', onDocumentMouseDown, false);
            document.addEventListener('mousemove', onDocumentMouseMove, false);
            document.addEventListener('mouseup', onDocumentMouseUp, false);
            document.addEventListener('mouseout', onDocumentMouseOut, false);
            window.addEventListener('resize', onWindowResize, false);
        }

        function onWindowResize() {
            windowHalfX = window.innerWidth / 2;
            windowHalfY = window.innerHeight / 2;
            camera.aspect = window.innerWidth / window.innerHeight;
            camera.updateProjectionMatrix();
            renderer.setSize(window.innerWidth, window.innerHeight);
        }

        function onDocumentMouseDown(event) {
            event.preventDefault();
            mousedown = true;
            mouseXOnMouseDown = event.clientX - windowHalfX;
            targetRotationOnMouseDown = targetRotation;
        }

        function onDocumentMouseMove(event) {
            if (!mousedown) return;
            mouseX = event.clientX - windowHalfX;
            targetRotation = targetRotationOnMouseDown + (mouseX - mouseXOnMouseDown) * 0.02;
            render();
        }

        function onDocumentMouseUp(event) {
            mousedown = false;
        }

        function onDocumentMouseOut(event) {
            mousedown = false;
        }

        function render() {
            plane.rotation.y = cube.rotation.y += (targetRotation - cube.rotation.y) * 0.05;
            renderer.render(scene, camera);
        }
";

            Interop.ExecuteJavaScript(script, div);

        }

    }
}
